import { CSSObject, DefaultTheme } from 'styled-components';

export const flexbox = <T extends { width?: number; theme: DefaultTheme }>({
  width,
  theme,
}: T) => {
  if (!width) {
    return;
  }

  const { breakpoints } = theme;

  const res: CSSObject = {
    flexBasis: '100%',
  };

  if (!Array.isArray(width)) {
    res['@media screen and (min-width: 64em)'] = {
      flexBasis: `${100 * width}%`,
    };

    return res;
  }

  for (let i = 0; i < width.length; i++) {
    if (breakpoints.length < i) {
      break;
    }

    const query = `@media screen and (min-width: ${breakpoints[i]})`;

    res[query] = {
      flexBasis: `${100 * width[i]}%`,
    };
  }

  return res;
};

const alignable = (isRow?: boolean) => <
  T extends { verticalAlign?: 'start' | 'center' | 'end'; centered?: boolean }
>({
  verticalAlign,
  centered,
}: T): CSSObject | null => {
  const s: Record<string, string | undefined> = {
    justifyContent: undefined,
    alignItems: undefined,
  };

  if (verticalAlign) {
    s[isRow ? 'alignItems' : 'justifyContent'] = verticalAlign;
  }

  if (centered) {
    s[!isRow ? 'alignItems' : 'justifyContent'] = 'center';
  }

  return s;
};

export const alignRow = alignable(true);
export const alignColumn = alignable();
